<?php

namespace Oro\Bundle\LocaleBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\ReversedTransformer;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToArrayTransformer;

use Oro\Bundle\FormBundle\Utils\FormUtils;

class QuarterType extends AbstractType
{
    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder->remove('year');

        FormUtils::replaceTransformer(
            $builder,
            new DateTimeToArrayTransformer($options['model_timezone'], $options['view_timezone'], ['month', 'day']),
            'view'
        );
        FormUtils::replaceTransformer(
            $builder,
            new ReversedTransformer(
                new DateTimeToArrayTransformer($options['model_timezone'], $options['model_timezone'], ['month', 'day'])
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            [
                'model_timezone' => 'UTC',
                'view_timezone'  => 'UTC',
                'format'         => 'dMMMy',
                'input'          => 'array',
            ]
        );

        $resolver->setAllowedValues(['input' => ['array']]);
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'oro_quarter';
    }

    /**
     * {@inheritdoc}
     */
    public function getParent()
    {
        return 'date';
    }
}
